#ifndef CPPVECTOR_H
#define CPPVECTOR_H
#include <string.h>
#define VECTOR_DEFCAP 8

template<typename type>
class Vector {
public:
	Vector(uint _capacity = VECTOR_DEFCAP) :
		fldsize(0),
		capacity(_capacity),
		array((type *) ourmalloc(sizeof(type) * _capacity)) {
	}

	Vector(uint _capacity, type *_array)  :
		fldsize(_capacity),
		capacity(_capacity),
		array((type *) ourmalloc(sizeof(type) * _capacity)) {
		memcpy(array, _array, capacity * sizeof(type));
	}

	Vector(Vector<type> *v) :
		fldsize(v->fldsize),
		capacity(v->capacity),
		array((type *) ourmalloc(sizeof(type) * v->capacity)) {
		memcpy(array, v->array, capacity * sizeof(type));
	}

	void pop() {
		fldsize--;
	}

	void remove(type t) {
		for (uint i = 0; i < fldsize; i++) {
			if (array[i] == t) {
				for (i++; i < fldsize; i++) {
					array[i - 1] = array[i];
				}
				fldsize--;
				break;
			}
		}
	}

	void removeIndex(uint i) {
		for (i++; i < fldsize; i++) {
			array[i - 1] = array[i];
		}
		fldsize--;
	}

	type last() const {
		return array[fldsize - 1];
	}

	void setSize(uint _size) {
		if (_size <= fldsize) {
			fldsize = _size;
			return;
		} else if (_size > capacity) {
			array = (type *)ourrealloc(array, _size * sizeof(type));
			capacity = _size;
		}
		bzero(&array[fldsize], (_size - fldsize) * sizeof(type));
		fldsize = _size;
	}

	void addAll(Vector<type> *v) {
		int oldsize = fldsize;
		setSize(fldsize + v->fldsize);
		memcpy(&array[fldsize], v->array, v->fldsize * sizeof(type));
	}

	void removeAll(Vector<type> *v) {
		uint vsize = v->size();
		for (uint i = 0; i < vsize; i++)
			remove(v->get(i));
	}

	void add(type item) {
		if (fldsize >= capacity) {
			uint newcap = capacity << 1;
			array = (type *)ourrealloc(array, newcap * sizeof(type));
			capacity = newcap;
		}
		array[fldsize++] = item;
	}

	type lastElement() {
		return get(size() - 1);
	}

	type firstElement() {
		return get(0);
	}

	type get(uint index) const {
		return array[index];
	}

	type setExpand(uint index, type item) {
		type retval = (type) 0;
		if (index >= fldsize)
			setSize(index + 1);
		else
			retval = array[index];
		set(index, item);
		return retval;
	}

	void set(uint index, type item) {
		array[index] = item;
	}

	uint size() const {
		return fldsize;
	}

	bool isEmpty() const {
		return fldsize == 0;
	}

	~Vector() {
		ourfree(array);
	}

	void clear() {
		fldsize = 0;
	}

	type *expose() {
		return array;
	}
	CMEMALLOC;
private:
	uint fldsize;
	uint capacity;
	type *array;
};
#endif
